###################################################
### chunk number 1: preliminaries
###################################################
library("bvpSolve")
options(prompt = "> ")
options(width=70)


###################################################
### chunk number 2: 
###################################################
fun<- function(x,y,pars) {
 list(c(y[2],
   1/ks*(-x*y[2]+y[1]-(1+ks*pi*pi)*cos(pi*x)-pi*x*sin(pi*x)))
     )
}


###################################################
### chunk number 3: 
###################################################
ks <- 0.1
x  <- seq(-1,1,by=0.01)
print(system.time(
sol1  <- bvpshoot(yini=c(-1,NA),yend=c(1,NA),x=x,func=fun,guess=0)
))

print(system.time(
sol2  <- bvptwp(yini=c(-1,NA),yend=c(1,NA),x=x,func=fun, guess=0)
))


###################################################
### chunk number 4: pr7a
###################################################
plot(sol2[,1],sol2[,3],type="l",main="test problem 7, ksi=0.1",
     lwd=2,col="red")
points(sol1[,1],sol1[,3],col="green",pch="x")
legend("topright",c("bvptwp", "bvpshoot"),
        lty=c(1,NA,NA), pch=c(NA,1,3),col=c("red", "green") )


###################################################
### chunk number 5: pr7a
###################################################
plot(sol2[,1],sol2[,3],type="l",main="test problem 7, ksi=0.1",
     lwd=2,col="red")
points(sol1[,1],sol1[,3],col="green",pch="x")
legend("topright",c("bvptwp", "bvpshoot"),
        lty=c(1,NA,NA), pch=c(NA,1,3),col=c("red", "green") )


###################################################
### chunk number 6: 
###################################################
ks <-0.0005

print(system.time(
sol3  <- bvptwp(yini=c(-1,NA),yend=c(1,NA),x=seq(-1,1,by=0.01),
           func=fun, xguess=sol2[,1], yguess=t(sol2[,-1]))
))


###################################################
### chunk number 7: pr7b
###################################################
plot(sol3,type="l", lwd=2,col="red")



###################################################
### chunk number 8: pr7b
###################################################
plot(sol3,type="l", lwd=2,col="red")



###################################################
### chunk number 9: 
###################################################
fsub <- function (t,Y,pars)  { 
  return(list(c(f1 = Y[2],
                f2 = (Y[1]*Y[4] - Y[3]*Y[2])/eps,
                f3 = Y[4],
                f4 = Y[5],
                f5 = Y[6],
                f6 = (-Y[3]*Y[6] - Y[1]*Y[2])/eps)))
}
eps <- 0.001
x <- seq(0,1,len=100)


###################################################
### chunk number 10: 
###################################################
print(system.time(
Soltwp <- bvptwp(x=x,func=fsub,guess= c(2,0,0),
                      yini=c(y1=-1,y2=NA,y3=0,y4=0,y5=NA,y6=NA),
                      yend=c(1,NA,0,0,NA,NA))
))


###################################################
### chunk number 11: swirl
###################################################
pairs(Soltwp, main="swirling flow III, eps=0.01", col="blue")


###################################################
### chunk number 12: swirl
###################################################
pairs(Soltwp, main="swirling flow III, eps=0.01", col="blue")


###################################################
### chunk number 13: 
###################################################
eps <- 0.0001
xguess <- Soltwp[,1]
yguess <- t(Soltwp[,2:7])


###################################################
### chunk number 14: 
###################################################
print(system.time(
  Sol2 <- bvptwp(x=x,func=fsub,guess= c(2,0,0),
  xguess=xguess,yguess=yguess,yini=c(y1=-1,y2=NA,y3=0,y4=0,y5=NA,y6=NA),
  yend=c(1,NA,0,0,NA,NA))
))


###################################################
### chunk number 15: swirl2
###################################################
plot(Sol2, col="darkred", type="l", lwd=2)
mtext(outer=TRUE, side=3, line=-1.5, cex=1.5,
   "swirling flow III, eps=0.0001")


###################################################
### chunk number 16: swirl2
###################################################
plot(Sol2, col="darkred", type="l", lwd=2)
mtext(outer=TRUE, side=3, line=-1.5, cex=1.5,
   "swirling flow III, eps=0.0001")


###################################################
### chunk number 17: 
###################################################
musn <- function(x,Y,pars) {
  with (as.list(Y),  {
    du=0.5*u*(w-u)/v
    dv=-0.5*(w-u)
    dw=(0.9-1000*(w-y)-0.5*w*(w-u))/z
    dz=0.5*(w-u)
    dy=-100*(y-w)
    return(list(c(du,dv,dw,dz,dy)))
  })
}


###################################################
### chunk number 18: 
###################################################
init <- c(u=1,v=1,w=1,z=-10,y=NA)


###################################################
### chunk number 19: 
###################################################
yend  <- function (Y,yini,pars)  with (as.list(Y), w-y)


###################################################
### chunk number 20: 
###################################################
print(system.time(
sol   <-bvpshoot(yini= init, x=seq(0,1,by=0.05),func=musn,
           yend=yend,guess=1,atol=1e-10,rtol=0)
))


###################################################
### chunk number 21: musn
###################################################
plot(sol,type="l", lwd=2)
mtext(outer=TRUE, side=3, line=-1.5, cex=1.5, "musn")


###################################################
### chunk number 22: musn
###################################################
plot(sol,type="l", lwd=2)
mtext(outer=TRUE, side=3, line=-1.5, cex=1.5, "musn")


###################################################
### chunk number 23: 
###################################################
bound <- function(i,y,pars) {
  with (as.list(y), {
    if (i ==1) return (u-1)
    if (i ==2) return (v-1)
    if (i ==3) return (w-1)
    if (i ==4) return (z+10)
    if (i ==5) return (w-y)
 })
}


###################################################
### chunk number 24: 
###################################################
xguess <- seq(0, 1, len=5)
yguess <- matrix(ncol = 5, (rep(c(1,1,1,-10,0.91),5)) )
rownames(yguess) <- c("u","v","w","z","y")
xguess
yguess


###################################################
### chunk number 25: 
###################################################
print(system.time(
Sol <- bvptwp(yini= NULL, x=x, func=musn, bound=bound,
              xguess=xguess, yguess=yguess, leftbc = 4,
              guess=1,atol=1e-10)
))


###################################################
### chunk number 26: 
###################################################
mathieu <- function(x,y,lambda)
    list(c(y[2],
        -(lambda-10*cos(2*x))*y[1]))


###################################################
### chunk number 27: 
###################################################
init <- c(1,0)
sol  <- bvpshoot(yini=init,yend=c(NA,0),x=seq(0,pi,by=0.01),
        func=mathieu, guess=NULL, extra=15)


###################################################
### chunk number 28: mat
###################################################
plot(sol[,1:2])
mtext(outer=TRUE, side=3, line=-1.5, cex=1.5, "mathieu")


###################################################
### chunk number 29: mat
###################################################
plot(sol[,1:2])
mtext(outer=TRUE, side=3, line=-1.5, cex=1.5, "mathieu")


###################################################
### chunk number 30: 
###################################################
attr(sol,"roots")  # root gives the value of "lambda" (17.10683)


###################################################
### chunk number 31: 
###################################################
mathieu2 <- function(x,y,p)
    list(c(y[2],
        -(y[3]-10*cos(2*x))*y[1],
        0) )


###################################################
### chunk number 32: 
###################################################
Sol <- bvptwp (yini= c(y=1,dy=0,lambda=NA), yend = c(NA,0,NA),
        x=seq(0,pi,by=0.01), func=mathieu2, xguess = c(0,pi),
        yguess = matrix(nr=3,rep(15,6)) )


###################################################
### chunk number 33: mat2
###################################################
plot(Sol, type = "l", lwd=2)
mtext(outer=TRUE, side=3, line=-1.5, cex=1.5, "mathieu - solved using bvptwp")


###################################################
### chunk number 34: mat2
###################################################
plot(Sol, type = "l", lwd=2)
mtext(outer=TRUE, side=3, line=-1.5, cex=1.5, "mathieu - solved using bvptwp")


###################################################
### chunk number 35: 
###################################################
nerve <- function (t,y,T)
  list(c( 3*T*(y[1] + y[2] - 1/3*(y[1]^3) - 1.3),
        (-1/3)*T*(y[1] - 0.7 + 0.8*y[2]) ))


###################################################
### chunk number 36: 
###################################################
res<- function (Y,yini,T)
  c(Y[1]-yini[1],
    Y[2]-yini[2],
    T*(-1/3)*(yini[1] - 0.7 + 0.8*yini[2]) - 1)


###################################################
### chunk number 37: 
###################################################
yini <- c(y1=NA,y2=NA)
sol  <- bvpshoot(yini=yini, x=seq(0,1,by=0.01),
        func=nerve, guess=c(0.5,0.5), yend=res, extra=2*pi)


###################################################
### chunk number 38: 
###################################################
attributes(sol)$root


###################################################
### chunk number 39: 
###################################################
nerve3 <- function (t,y,p)
  list(c( 3*y[3]*(y[1] + y[2] - 1/3*(y[1]^3) - 1.3),
        (-1/3)*y[3]*(y[1] - 0.7 + 0.8*y[2]) ,
        0,
        0,
        0)
  )


###################################################
### chunk number 40: 
###################################################
bound <- function(i,y,p) {
 if (i ==1) return ( y[3]*(-1/3)*(y[1] - 0.7 + 0.8*y[2]) - 1 )
 if (i ==2) return ( y[1]-y[4] )           
 if (i ==3) return ( y[2]-y[5] )           # left bnd
 if (i ==4) return ( y[1]-y[4] )           # right bnd
 if (i ==5) return ( y[2]-y[5] )
}


###################################################
### chunk number 41: 
###################################################
xguess = seq(0,1,by=0.1)
yguess = matrix(nr=5,nc=length(xguess),5.)
yguess[1,] <- sin(2*pi*xguess)
yguess[2,] <- cos(2*pi*xguess)
rownames(yguess) <- c("y1", "y2", "T", "y1ini", "y2ini")


###################################################
### chunk number 42: 
###################################################
Sol  <- bvptwp(func=nerve3, bound=bound, x=seq(0,1,by=0.01), 
        guess = c(y=0.5, dy=0.5, T=2*pi, yi=0.5, yj=0.5),
        leftbc=3, xguess=xguess, yguess=yguess)


###################################################
### chunk number 43: 
###################################################
Sol[1,]


###################################################
### chunk number 44: nerve
###################################################
plot(Sol, type="l", lwd=2, col="darkblue")


###################################################
### chunk number 45: nerve
###################################################
plot(Sol, type="l", lwd=2, col="darkblue")


###################################################
### chunk number 46: 
###################################################
Elastica <- function (x, y, pars) {

  list( c(cos(y[3]),
          sin(y[3]),
          y[4],
          y[5]*cos(y[3]),
          0))
}

Sol <- bvptwp(func=Elastica,
              yini = c(x=0, y=0, p=NA,   k=0, F=NA),
              yend = c(x=NA,y=0, p=-pi/2,k=NA,F=NA),
              x = seq(0,0.5,len=16),
              guess=c(0,0) )


###################################################
### chunk number 47: elastica
###################################################
plot(Sol)


###################################################
### chunk number 48: elastica
###################################################
plot(Sol)


###################################################
### chunk number 49: 
###################################################


jacfunc <- function (x, y, pars) {
      Jac <- matrix(nrow=5, ncol=5, 0)
      Jac[3,4]=1.0
      Jac[4,4]=1.0
      Jac[1,3]=-sin(y[3])
      Jac[2,3]=cos(y[3])
      Jac[4,3]=-y[5]*sin(y[3])
      Jac[4,5]=Jac[2,3]
      Jac
}


###################################################
### chunk number 50: 
###################################################
bound <- function (i, y, pars)  {
    if (i <=2) return(y[i])
    else if (i == 3) return(y[4])
    else if (i == 4) return(y[2])
    else if (i == 5) return(y[3]+pi/2)
}


###################################################
### chunk number 51: 
###################################################
jacbound <- function(i, y, pars)  {
    JJ <- rep(0,5)
         if (i <=2) JJ[i] =1.0
    else if (i ==3) JJ[4] =1.0
    else if (i ==4) JJ[2] =1.0
    else if (i ==5) JJ[3] =1.0
    JJ
}


###################################################
### chunk number 52: 
###################################################
Sol4 <- bvptwp(leftbc = 3,
              func=Elastica, jacfunc = jacfunc,
              bound = bound, jacbound = jacbound,
              x = seq(0,0.5,len=16),
              guess=c(0,0) )


###################################################
### chunk number 53: 
###################################################
outF <- bvptwp(ncomp=5,
               x = seq(0,0.5,len=16), leftbc = 3,
               func="fsub",jacfunc="dfsub",bound="gsub",jacbound="dgsub",
               dllname="bvpSolve")


###################################################
### chunk number 54: 
###################################################
fun <- function(t,y,pars)
  list(c( y[2],
        - a*p*y[1]/(p+t*t)^2
        ))


###################################################
### chunk number 55: 
###################################################
p    <- 1e-5
a    <- 3


###################################################
### chunk number 56: 
###################################################
sol  <- bvptwp(yini = c(y=-0.1/sqrt(p+0.01), dy=NA),
               yend = c(   0.1/sqrt(p+0.01),    NA),
               x = seq(-0.1, 0.1, by=0.001),
               func = fun, guess = 1)


###################################################
### chunk number 57: linear
###################################################
plot(sol,type="l")


###################################################
### chunk number 58: linear
###################################################
plot(sol,type="l")


###################################################
### chunk number 59: 
###################################################
parms <- c(a=3, p=1e-7)


###################################################
### chunk number 60: 
###################################################
Out  <- NULL
x    <- seq(-0.1,0.1,by=0.001)
pseq <- 10^-seq(0,6,0.5)

for (pp in pseq) {
  parms[2] <- pp
  outFor <- bvptwp(ncomp=2,
               x = x, leftbc = 1, initfunc="initbnd", parms=parms, guess=1,
               func="funbnd",jacfunc="dfbnd",bound="gbnd",jacbound="dgbnd",
               allpoints=FALSE,dllname="bvpSolve")
  Out <- cbind(Out, outFor[,2])
}


###################################################
### chunk number 62: linf
###################################################
matplot(x,Out,type="l")
legend("topleft", legend=log10(pseq), title="logp",
  col=1:length(pseq), lty=1:length(pseq), cex=0.6)


###################################################
### chunk number 63: linf
###################################################
matplot(x,Out,type="l")
legend("topleft", legend=log10(pseq), title="logp",
  col=1:length(pseq), lty=1:length(pseq), cex=0.6)


